package kr.co.vividnext.sodalive.audio_content.main.v2.content

import android.annotation.SuppressLint
import android.content.Intent
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.view.View
import android.widget.LinearLayout
import android.widget.Toast
import androidx.annotation.OptIn
import androidx.core.content.ContextCompat
import androidx.media3.common.util.UnstableApi
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.zhpan.bannerview.BaseBannerAdapter
import com.zhpan.indicator.enums.IndicatorSlideMode
import com.zhpan.indicator.enums.IndicatorStyle
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audio_content.all.AudioContentNewAllActivity
import kr.co.vividnext.sodalive.audio_content.all.AudioContentNewAllAdapter
import kr.co.vividnext.sodalive.audio_content.detail.AudioContentDetailActivity
import kr.co.vividnext.sodalive.audio_content.main.AudioContentBannerType
import kr.co.vividnext.sodalive.audio_content.main.AudioContentMainContentAdapter
import kr.co.vividnext.sodalive.audio_content.main.banner.AudioContentMainBannerAdapter
import kr.co.vividnext.sodalive.audio_content.main.new_content.AudioContentMainNewContentThemeAdapter
import kr.co.vividnext.sodalive.audio_content.main.ranking.AudioContentMainRankingAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.AudioContentMainContentCurationAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.ContentRankCreatorAdapter
import kr.co.vividnext.sodalive.audio_content.main.v2.PopularContentByCreatorAdapter
import kr.co.vividnext.sodalive.audio_content.series.detail.SeriesDetailActivity
import kr.co.vividnext.sodalive.base.BaseFragment
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.GridSpacingItemDecoration
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.databinding.FragmentAudioContentMainTabContentBinding
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.live.event_banner.EventBannerAdapter
import kr.co.vividnext.sodalive.settings.event.EventDetailActivity
import org.koin.android.ext.android.inject
import kotlin.math.roundToInt

@OptIn(UnstableApi::class)
class AudioContentMainTabContentFragment : BaseFragment<FragmentAudioContentMainTabContentBinding>(
    FragmentAudioContentMainTabContentBinding::inflate
) {
    private val viewModel: AudioContentMainTabContentViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var contentBannerAdapter: AudioContentMainBannerAdapter
    private lateinit var newContentThemeAdapter: AudioContentMainNewContentThemeAdapter
    private lateinit var newContentAdapter: AudioContentMainContentAdapter
    private lateinit var contentRankingSortAdapter: AudioContentMainNewContentThemeAdapter
    private lateinit var contentRankingAdapter: AudioContentMainRankingAdapter
    private lateinit var contentRankCreatorAdapter: ContentRankCreatorAdapter
    private lateinit var curationAdapter: AudioContentMainContentCurationAdapter
    private lateinit var popularContentByCreatorAdapter: PopularContentByCreatorAdapter
    private lateinit var contentTagAdapter: AudioContentMainTabContentTagAdapter
    private lateinit var contentByTagAdapter: AudioContentNewAllAdapter

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setupView()
        bindData()

        viewModel.fetchData()
    }

    private fun setupView() {
        loadingDialog = LoadingDialog(requireActivity(), layoutInflater)

        setupContentBanner()
        setupNewContentTheme()
        setupNewContent()
        setupContentRankingSortType()
        setupContentRanking()
        setupEventBanner()
        setupPopularContentCreator()
        setupPopularContentByCreator()
        setupContentTag()
        setupContentByTag()
        setupCuration()
    }

    private fun setupContentBanner() {
        val layoutParams = binding
            .rvBanner
            .layoutParams as LinearLayout.LayoutParams

        val pagerWidth = screenWidth.toDouble() - 26.7f.dpToPx()
        val pagerHeight = (pagerWidth * 0.53).roundToInt()
        layoutParams.width = pagerWidth.roundToInt()
        layoutParams.height = pagerHeight

        contentBannerAdapter = AudioContentMainBannerAdapter(
            requireContext(),
            pagerWidth.roundToInt(),
            pagerHeight
        ) {
            when (it.type) {
                AudioContentBannerType.EVENT -> {
                    startActivity(
                        Intent(requireContext(), EventDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_EVENT, it.eventItem!!)
                        }
                    )
                }

                AudioContentBannerType.CREATOR -> {
                    startActivity(
                        Intent(requireContext(), UserProfileActivity::class.java).apply {
                            putExtra(Constants.EXTRA_USER_ID, it.creatorId!!)
                        }
                    )
                }

                AudioContentBannerType.SERIES -> {
                    startActivity(
                        Intent(requireContext(), SeriesDetailActivity::class.java).apply {
                            putExtra(Constants.EXTRA_SERIES_ID, it.seriesId!!)
                        }
                    )
                }

                AudioContentBannerType.LINK -> {
                    startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(it.link!!)))
                }
            }
        }

        binding
            .rvBanner
            .layoutParams = layoutParams

        binding.rvBanner.apply {
            adapter = contentBannerAdapter as BaseBannerAdapter<Any>

            setLifecycleRegistry(lifecycle)
            setScrollDuration(1000)
            setInterval(4 * 1000)
        }.create()

        binding
            .rvBanner
            .setIndicatorView(binding.indicatorBanner)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_3bb9f1)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.contentBannerLiveData.observe(viewLifecycleOwner) {
            if (contentBannerAdapter.itemCount <= 0 && it.isEmpty()) {
                binding.rvBanner.visibility = View.GONE
                binding.indicatorBanner.visibility = View.GONE
            } else {
                binding.rvBanner.visibility = View.VISIBLE
                binding.indicatorBanner.visibility = View.VISIBLE
                binding.rvBanner.refreshData(it)
            }
        }
    }

    private fun setupNewContentTheme() {
        newContentThemeAdapter = AudioContentMainNewContentThemeAdapter {
            viewModel.getNewContentOfTheme(theme = it)
        }

        binding.rvNewContentTheme.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvNewContentTheme.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 4f.dpToPx().toInt()
                    }

                    newContentThemeAdapter.itemCount - 1 -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 4f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvNewContentTheme.adapter = newContentThemeAdapter

        viewModel.themeListLiveData.observe(viewLifecycleOwner) {
            binding.llNewContent.visibility = View.VISIBLE
            newContentThemeAdapter.addItems(it)
        }
    }

    private fun setupNewContent() {
        binding.ivNewContentAll.setOnClickListener {
            startActivity(Intent(requireContext(), AudioContentNewAllActivity::class.java))
        }

        newContentAdapter = AudioContentMainContentAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        binding.rvNewContent.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvNewContent.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 6.7f.dpToPx().toInt()
                    }

                    newContentAdapter.itemCount - 1 -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 6.7f.dpToPx().toInt()
                        outRect.right = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvNewContent.adapter = newContentAdapter

        viewModel.newContentListLiveData.observe(viewLifecycleOwner) {
            newContentAdapter.addItems(it)
        }
    }

    private fun setupContentRankingSortType() {
        contentRankingSortAdapter = AudioContentMainNewContentThemeAdapter {
            viewModel.getContentRanking(sort = it)
        }

        binding.rvContentRankingSort.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvContentRankingSort.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 4f.dpToPx().toInt()
                    }

                    contentRankingSortAdapter.itemCount - 1 -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 4f.dpToPx().toInt()
                        outRect.right = 4f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvContentRankingSort.adapter = contentRankingSortAdapter

        viewModel.contentRankingSortListLiveData.observe(viewLifecycleOwner) {
            binding.llContentRanking.visibility = View.VISIBLE
            contentRankingSortAdapter.addItems(it)
        }
    }

    @SuppressLint("SetTextI18n")
    private fun setupContentRanking() {
        contentRankingAdapter = AudioContentMainRankingAdapter(
            width = (screenWidth * 0.66).toInt()
        ) {
            startActivity(
                Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                    putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                }
            )
        }

        binding.rvContentRanking.layoutManager = GridLayoutManager(
            context,
            3,
            GridLayoutManager.HORIZONTAL,
            false
        )

        binding.rvContentRanking.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
            }
        })

        binding.rvContentRanking.adapter = contentRankingAdapter

        viewModel.contentRankingLiveData.observe(viewLifecycleOwner) {
            binding.llContentRanking.visibility = View.VISIBLE
            contentRankingAdapter.addItems(it)
        }
    }

    private fun setupEventBanner() {
        val imageSliderLp = binding.eventBannerSlider.layoutParams
        imageSliderLp.width = screenWidth
        imageSliderLp.height = (screenWidth * 300) / 1000
        binding.eventBannerSlider.layoutParams = imageSliderLp

        binding.eventBannerSlider.apply {
            adapter = EventBannerAdapter(requireContext()) {
                if (it.detailImageUrl != null) {
                    val intent = Intent(requireActivity(), EventDetailActivity::class.java)
                    intent.putExtra(Constants.EXTRA_EVENT, it)
                    startActivity(intent)
                } else if (!it.link.isNullOrBlank()) {
                    startActivity(
                        Intent(
                            Intent.ACTION_VIEW,
                            Uri.parse(it.link)
                        )
                    )
                }
            } as BaseBannerAdapter<Any>
            setLifecycleRegistry(lifecycle)
            setScrollDuration(800)
        }.create()

        binding.eventBannerSlider
            .setIndicatorView(binding.indicatorEventBanner)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_3bb9f1)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.eventLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.eventBannerSlider.visibility = View.VISIBLE
                binding.indicatorEventBanner.visibility = View.VISIBLE
                binding.eventBannerSlider.refreshData(it)
            } else {
                binding.eventBannerSlider.visibility = View.GONE
                binding.indicatorEventBanner.visibility = View.GONE
            }
        }
    }

    private fun setupPopularContentCreator() {
        contentRankCreatorAdapter = ContentRankCreatorAdapter {
            binding.llNoItems.visibility = View.VISIBLE
            binding.rvRankingSalesCount.visibility = View.GONE
            viewModel.getPopularContentByCreator(it)
        }

        binding.rvRankingCreator.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding.rvRankingCreator.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 0
                        outRect.right = 11f.dpToPx().toInt()
                    }

                    contentRankCreatorAdapter.itemCount - 1 -> {
                        outRect.left = 11f.dpToPx().toInt()
                        outRect.right = 0
                    }

                    else -> {
                        outRect.left = 11f.dpToPx().toInt()
                        outRect.right = 11f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvRankingCreator.adapter = contentRankCreatorAdapter

        viewModel.contentRankCreatorListLiveData.observe(viewLifecycleOwner) {
            contentRankCreatorAdapter.addItems(it)
            if (contentRankCreatorAdapter.itemCount <= 0 && it.isEmpty()) {
                binding.llCreatorContentRanking.visibility = View.GONE
            } else {
                binding.llCreatorContentRanking.visibility = View.VISIBLE
            }
        }
    }

    private fun setupPopularContentByCreator() {
        popularContentByCreatorAdapter = PopularContentByCreatorAdapter(
            itemWidth = ((screenWidth - 13.3f.dpToPx() * 3) / 2).toInt(),
            onClickItem = { contentId ->
                startActivity(
                    Intent(requireActivity(), AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, contentId)
                    }
                )
            },
            onClickCreator = { creatorId ->
                startActivity(
                    Intent(requireActivity(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, creatorId)
                    }
                )
            }
        )

        val recyclerView = binding.rvRankingSalesCount
        recyclerView.layoutManager = GridLayoutManager(requireContext(), 2)
        recyclerView.addItemDecoration(
            GridSpacingItemDecoration(
                2,
                13.3f.dpToPx().toInt(),
                false
            )
        )

        recyclerView.adapter = popularContentByCreatorAdapter

        viewModel.salesCountRankContentListLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.llNoItems.visibility = View.GONE
                recyclerView.visibility = View.VISIBLE
                popularContentByCreatorAdapter.addItems(it)
            } else {
                binding.llNoItems.visibility = View.VISIBLE
                recyclerView.visibility = View.GONE
            }
        }
    }

    private fun setupContentTag() {
        val spanCount = 4
        val spacing = 6f.dpToPx()

        contentTagAdapter = AudioContentMainTabContentTagAdapter {
            viewModel.getRecommendContentByTag(it)
        }

        val recyclerView = binding.rvRecommendContentTag
        recyclerView.layoutManager = GridLayoutManager(requireContext(), spanCount)
        recyclerView.addItemDecoration(GridSpacingItemDecoration(spanCount, spacing.toInt(), false))
        recyclerView.adapter = contentTagAdapter

        viewModel.tagListLiveData.observe(viewLifecycleOwner) {
            contentTagAdapter.addItems(it)
            if (
                contentTagAdapter.itemCount <= 0 ||
                !SharedPreferenceManager.isAdultContentVisible ||
                !SharedPreferenceManager.isAuth
            ) {
                binding.llRecommendContentByTag.visibility = View.GONE
            } else {
                binding.llRecommendContentByTag.visibility = View.VISIBLE
            }
        }
    }

    private fun setupContentByTag() {
        val spanCount = 3
        val horizontalSpacing = 13.3f.dpToPx().toInt()
        val verticalSpacing = 26.7f.dpToPx().toInt()
        val itemWidth = (screenWidth - horizontalSpacing * (spanCount + 1)) / spanCount
        contentByTagAdapter = AudioContentNewAllAdapter(
            itemWidth = itemWidth,
            onClickItem = {
                startActivity(
                    Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        val recyclerView = binding.rvRecommendContent
        recyclerView.layoutManager = GridLayoutManager(requireContext(), spanCount)
        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                outRect.left = horizontalSpacing / 2
                outRect.right = horizontalSpacing / 2
                outRect.top = verticalSpacing / 2
                outRect.bottom = verticalSpacing / 2
            }
        })
        recyclerView.adapter = contentByTagAdapter

        viewModel.tagCurationContentListLiveData.observe(viewLifecycleOwner) {
            contentByTagAdapter.clear()
            contentByTagAdapter.addItems(it)
        }
    }

    private fun setupCuration() {
        curationAdapter = AudioContentMainContentCurationAdapter(
            onClickItem = {
                startActivity(
                    Intent(requireContext(), AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, it)
                    }
                )
            },
            onClickCreator = {
                startActivity(
                    Intent(requireContext(), UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, it)
                    }
                )
            }
        )

        binding.rvCuration.layoutManager = LinearLayoutManager(
            context,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvCuration.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 30f.dpToPx().toInt()
                        outRect.bottom = 15f.dpToPx().toInt()
                    }

                    curationAdapter.itemCount - 1 -> {
                        outRect.top = 15f.dpToPx().toInt()
                        outRect.bottom = 30f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.top = 15f.dpToPx().toInt()
                        outRect.bottom = 15f.dpToPx().toInt()
                    }
                }
            }
        })

        binding.rvCuration.adapter = curationAdapter

        viewModel.curationListLiveData.observe(viewLifecycleOwner) {
            curationAdapter.addItems(it)

            binding.rvCuration.visibility = if (curationAdapter.itemCount <= 0 && it.isEmpty()) {
                View.GONE
            } else {
                View.VISIBLE
            }
        }
    }

    private fun bindData() {
        viewModel.toastLiveData.observe(viewLifecycleOwner) {
            it?.let { Toast.makeText(requireContext(), it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(viewLifecycleOwner) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }
    }
}
