package kr.co.vividnext.sodalive.base

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.ActivityInfo
import android.os.Bundle
import android.view.LayoutInflater
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import androidx.viewbinding.ViewBinding
import io.reactivex.rxjava3.disposables.CompositeDisposable

abstract class BaseActivity<T : ViewBinding>(
    private val inflate: (LayoutInflater) -> T
) : AppCompatActivity() {

    val compositeDisposable = CompositeDisposable()

    lateinit var binding: T
        private set

    val screenWidth: Int by lazy {
        resources.displayMetrics.widthPixels
    }

    val screenHeight: Int by lazy {
        resources.displayMetrics.heightPixels
    }

    private val pointReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val message = intent?.getStringExtra("message") ?: return
            showPointDialog(message)
        }
    }

    @SuppressLint("SourceLockedOrientationActivity")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT

        binding = inflate(layoutInflater)
        setContentView(binding.root)

        setupView()
    }

    override fun onStart() {
        super.onStart()
        LocalBroadcastManager.getInstance(this)
            .registerReceiver(pointReceiver, IntentFilter("ACTION_POINT_GRANTED"))
    }

    override fun onStop() {
        super.onStop()
        LocalBroadcastManager.getInstance(this).unregisterReceiver(pointReceiver)
    }

    override fun onDestroy() {
        compositeDisposable.clear()
        super.onDestroy()
    }

    private fun showPointDialog(message: String) {
        SodaDialog(
            activity = this,
            layoutInflater = layoutInflater,
            title = "포인트 지급",
            desc = message,
            confirmButtonTitle = "확인",
            confirmButtonClick = {}
        ).show(screenWidth)
    }

    fun showToast(message: String) {
        Toast.makeText(applicationContext, message, Toast.LENGTH_LONG).show()
    }

    abstract fun setupView()
}
