package kr.co.vividnext.sodalive.search

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import coil.transform.RoundedCornersTransformation
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.databinding.ItemSearchBinding
import kr.co.vividnext.sodalive.extensions.dpToPx

class SearchAdapter(
    private val onClickItem: (SearchResponseItem) -> Unit
) : RecyclerView.Adapter<SearchAdapter.ViewHolder>() {
    inner class ViewHolder(
        private val context: Context,
        private val binding: ItemSearchBinding
    ) : RecyclerView.ViewHolder(binding.root) {

        @SuppressLint("CheckResult")
        fun bind(item: SearchResponseItem) {
            binding.ivProfile.visibility = View.GONE
            binding.ivContent.visibility = View.GONE
            binding.ivSeries.visibility = View.GONE
            binding.tvNickname.visibility = View.GONE

            when (item.type) {
                SearchResponseType.CREATOR -> {
                    binding.ivProfile.visibility = View.VISIBLE
                    binding.ivProfile.load(item.imageUrl) {
                        crossfade(true)
                        placeholder(R.drawable.bg_placeholder)
                        transformations(CircleCropTransformation())
                    }
                }

                SearchResponseType.CONTENT -> {
                    binding.ivContent.visibility = View.VISIBLE
                    binding.ivContent.load(item.imageUrl) {
                        crossfade(true)
                        placeholder(R.drawable.bg_placeholder)
                        transformations(RoundedCornersTransformation(5.3f.dpToPx()))
                    }
                    binding.tvNickname.visibility = View.VISIBLE
                }

                SearchResponseType.SERIES -> {
                    binding.ivSeries.visibility = View.VISIBLE
                    binding.ivSeries.load(item.imageUrl) {
                        crossfade(true)
                        placeholder(R.drawable.bg_placeholder)
                        transformations(RoundedCornersTransformation(5.3f.dpToPx()))
                    }

                    binding.ivSeries.load(item.imageUrl) {
                        crossfade(true)
                        placeholder(R.drawable.bg_placeholder)
                        transformations(RoundedCornersTransformation(5.3f.dpToPx()))
                    }
                    binding.tvNickname.visibility = View.VISIBLE
                }
            }

            binding.tvTitle.text = item.title
            binding.tvNickname.text = item.nickname

            binding.root.setOnClickListener { onClickItem(item) }
        }
    }

    val items: MutableList<SearchResponseItem> = mutableListOf()

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        parent.context,
        ItemSearchBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(items[position])
    }

    override fun getItemCount() = items.size

    @SuppressLint("NotifyDataSetChanged")
    fun clear() {
        items.clear()
        notifyDataSetChanged()
    }
}
